class GeneralFunctionsQueryBuilder {
  constructor(elements) {
    this.elements = elements;
  }

  toggleClass(elements, className, action){
    elements.forEach(element => {
      document.querySelector('[data-toggle-area="' + element + '"]').classList[action](className);
    });
  }

  queryChipFactory(dataType, prettyQueryText, queryText) {
    this.elements.counter++;
    this.elements.queryInputFieldInstance.addChip({
      tag: prettyQueryText
    });

    let queryChipElementIndex = this.elements.queryInputFieldInstance.$chips.length - 1;
    let queryChipElement = this.elements.queryInputFieldInstance.$chips[queryChipElementIndex];
    queryChipElement.classList.add('query-component');
    queryChipElement.setAttribute('data-type', dataType);
    queryChipElement.setAttribute('data-query', queryText);
    queryChipElement.setAttribute('draggable', 'true');

    queryChipElement.addEventListener('click', () => this.deleteAttr(queryChipElement));
    queryChipElement.addEventListener('dragstart', this.handleDragStart.bind(this, queryChipElement));
    queryChipElement.addEventListener('dragend', this.handleDragEnd);

    this.queryPreviewBuilder();
  }

  handleDragStart(queryChipElement, event) {
    let queryChips = this.elements.queryInputField.querySelectorAll('.query-component');
    setTimeout(() => {
      let targetChipElement = Utils.HTMLToElement('<span class="chip drop-target">Drop here</span>');
      for (let element of queryChips) {
        if (element === queryChipElement.nextSibling) {continue;}
        let targetChipClone = targetChipElement.cloneNode(true);
        if (element === queryChipElement && queryChips[queryChips.length - 1] !== element) {
          queryChips[queryChips.length - 1].insertAdjacentElement('afterend', targetChipClone);
        } else {
          element.insertAdjacentElement('beforebegin', targetChipClone);
        }
        this.addDragDropListeners(targetChipClone, queryChipElement);
      }
    }, 0);
  }

  handleDragEnd(event) {
    document.querySelectorAll('.drop-target').forEach(target => target.remove());
  }

  addDragDropListeners(targetChipClone, queryChipElement) {
    targetChipClone.addEventListener('dragover', (event) => {
      event.preventDefault();
    });
    targetChipClone.addEventListener('dragenter', (event) => {
      event.preventDefault();
      event.target.style.borderStyle = 'solid dotted';
    });
    targetChipClone.addEventListener('dragleave', (event) => {
      event.preventDefault();
      event.target.style.borderStyle = 'hidden';
    });
    targetChipClone.addEventListener('drop', (event) => {
      let dropzone = event.target;
      dropzone.parentElement.replaceChild(queryChipElement, dropzone);
      this.elements.queryInputFieldInstance.$chips = Array.from(this.elements.queryInputField.querySelectorAll('.chip'));
      this.queryPreviewBuilder();
    });
  }

  queryPreviewBuilder() {
    let queryPreview = document.querySelector('#corpus-analysis-concordance-query-preview');
    let queryChipElements = Array.from(Object.values(this.elements.queryInputFieldInstance.$chips));
    if (!isNaN(queryChipElements[queryChipElements.length - 1])) {
      queryChipElements.pop();
    }
    this.elements.queryInputFieldContent = [];
    queryChipElements.forEach(element => {

      let queryElement = element.dataset.query;
      if (queryElement !== undefined) {
        queryElement = Utils.escape(queryElement);
        this.elements.queryInputFieldContent.push(queryElement);
      }
    });
  
    let queryString = this.elements.queryInputFieldContent.join(' ');
    queryString += ';';
  
    queryPreview.innerHTML = queryString;
    queryPreview.parentNode.classList.toggle('hide', queryString === ';');
  }

  deleteAttr(attr) {
    if (attr.dataset.type === "start-sentence") {
      this.elements.sentenceElement.innerHTML = 'Sentence';
    } else if (attr.dataset.type === "start-entity" || attr.dataset.type === "start-empty-entity") {
      this.elements.entityElement.innerHTML = 'Entity';
    }
    let queryChipElements = Array.from(Object.values(this.elements.queryInputFieldInstance.$chips));
    queryChipElements.pop();
    this.elements.counter -= 1;
    this.elements.queryInputFieldInstance.deleteChip(queryChipElements.indexOf(attr));
    this.queryPreviewBuilder();
  }
}

